

    /* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *\
    *                                                                 *
    *   DEVIL Screensaver System 1.05                                 *
    *   Copyright  2005 Sebastian Pipping <webmaster@hartwork.org>   *
    *                                                                 *
    *   --> http://www.hartwork.org                                   *
    *                                                                 *
    *                                                                 *
    *   This source code is released under LGPL.                      *
    *   See LGPL.txt for details.                        2005-09-18   *
    *                                                                 *
    \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */


/**
 *  Marching Cubes Demo, by glUser3f [glUser3f@gamedev.net]
 *  Feel free to use this code in your own projects.
 *  If you do something cool with it, please email me so I can see it!
 *
 *  Credits:
 *  Theory by Paul Bourke        [http://astronomy.swin.edu.au/~pbourke/modelling/polygonise/]
 *  OpenGL basecode by NeHe      [nehe.gamedev.net]
 *  Lookup Tables by Paul Bourke [http://astronomy.swin.edu.au/~pbourke/modelling/polygonise/]
 *  The rest is done by me, glUser3f ;)
 *
 */


/*

	-- COMPILE INFO --
	libs:		OPENGL32.LIB GLU32.LIB MSVCRT.LIB kernel32.lib user32.lib gdi32.lib
	threads:	multi-threaded
	opts:		disable all default libs

*/



#include "Devil.h"
#include "DevilConfig.h"
#include <gl/gl.h>
#include <gl/glu.h>
#include <stdlib.h>
#include "glfw.h"



// Don't warn on int->bool
#pragma warning( disable : 4800 )


#ifndef __min
# define __min( a,b ) ( a < b ? a : b )
#endif

#ifndef __max
# define __max( a,b ) ( a > b ? a : b )
#endif


// Imported from glfw.cpp
extern float AmbLight[ 4 ];
extern float DifLight[ 4 ];
extern float LightPos[ 4 ];

extern bool gWire;
extern bool gLight;
extern bool gPaused;

extern float zoom;

extern bool g_bPriorDown;
extern bool g_bNextDown;


extern BOOL glfwActive;

extern HDC hDC;



bool FACE_Live();
void FACE_Die();
LRESULT CALLBACK FACE_Master( HWND, UINT, WPARAM, LPARAM );
void FACE_Render( const double );
void FACE_Config( const HWND );
bool FACE_Init3D();
void FACE_Cleanup3D();
// bool FACE_SwitchRes( int *, int *, UINT *, UINT * );
// void FACE_RestoreRes();



DEVIL_CONFIG g_CONFIG;

DEVIL_INTERFACE g_FACE =
{
	FACE_Live,				// Live
	FACE_Die,				// Die
	FACE_Master,			// Master
	FACE_Render,			// Render
	FACE_Config,			// Config
	FACE_Init3D,			// Init3D
	FACE_Cleanup3D,			// Cleanup3D
	NULL,					// SwitchRes
	NULL,					// RestoreRes
	false,					// bEnableFrameLimiter
	0,						// uMaxFPS
	true,					// bMouseExit
	false,					// bAnyKeyExit

	NULL,					// hwnd
	DEVIL_MODE_UNKNOWN,		// Mode
	NULL,					// Slave
	0.0						// fFPS
};



// ***************************************************************************************
bool FACE_Live()
{
	// Read config
	int iWire = 0;
	g_CONFIG.Read( TEXT( "Wireframe" ), &iWire, iWire );
	gWire = ( bool )iWire;

	int iLight = 1;
	g_CONFIG.Read( TEXT( "Light" ), &iLight, iLight );
	gLight = ( bool )iLight;

	double fZoom = 2.2;
	g_CONFIG.Read( TEXT( "Zoom" ), &fZoom, fZoom );
	zoom = __max( 1.3f, __min( ( float )fZoom, 30.0f ) );

	return true;
}



// ***************************************************************************************
void FACE_Die()
{
	// Write config
	g_CONFIG.Write( TEXT( "Wireframe" ), ( int )gWire );
	g_CONFIG.Write( TEXT( "Light" ), ( int )gLight );
	g_CONFIG.Write( TEXT( "Zoom" ), ( double )zoom );
}



// ***************************************************************************************
LRESULT CALLBACK FACE_Master( HWND hwnd, UINT message, WPARAM wp, LPARAM lp )
{
	switch( message )
	{
	case WM_ACTIVATE:
		glfwActive = !HIWORD( wp );
		return 0;

	case WM_SIZE:
		glfwResizeScene( LOWORD( lp ), HIWORD( lp ) );
		return 0;

	case WM_KEYDOWN:
		switch( wp )
		{
		case VK_PRIOR:
			g_bPriorDown = true;
			break;

		case VK_NEXT:
			g_bNextDown = true;
			break;

		case 'W':
			// Down before?
			if( lp & 0x40000000 ) break;

			gWire = !gWire;
			if (gWire)
			{
				glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
			}
			else
			{
				glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
			}
			break;

		case 'L':
			// Down before?
			if( lp & 0x40000000 ) break;

			gLight = !gLight;
			if (gLight)
			{
				glEnable(GL_LIGHTING);
			}
			else
			{
				glDisable(GL_LIGHTING);
			}
			break;

		case 'P':
			// Down before?
			if( lp & 0x40000000 ) break;

			gPaused = !gPaused;
			break;

		}
		break;

	case WM_KEYUP:
		switch( wp )
		{
		case VK_PRIOR:
			g_bPriorDown = false;
			break;

		case VK_NEXT:
			g_bNextDown = false;
			break;

		}
		break;

	}
	return CallWindowProc( g_FACE.Slave, hwnd, message, wp, lp );
}



// ***************************************************************************************
void FACE_Render( const double fPassedTime )
{
	if( glfwActive )
	{
		glfwRender();
		glfwUpdate( fPassedTime );
		SwapBuffers( hDC );
	}
}



// ***************************************************************************************
void FACE_Config( const HWND hParent )
{
	MessageBox(
		hParent,
		TEXT(	"Welcome to the Marching Cubes Algorithm Demo.  \n"
				"\n"
				"Programmed by...\n"
				"* glUser3f <glUser3f@gamedev.net>\n"
				"\n"
				"Screensaver conversion by...\n"
				"* Sebastian Pipping <webmaster@hartwork.org>  \n"
				"\n"
				"\n"
				"   -- Controls --\n"
				"Pg up\t Zoom in\n"
				"Pg down\t Zoom out\n"
				"W\t Wireframe/polygons\n"
				"L\t Light on/off\n"
				"P\t Pause\n\n\nEnjoy!" ),
		TEXT( "Marching Cubes Screensaver 1.33" ),
		0
	);
}



// ***************************************************************************************
bool FACE_Init3D()
{
	bool res = ( bool )glfwCreateWnd( g_FACE.hwnd );

	if( res )
	{
		if( gWire )
		{
			glPolygonMode( GL_FRONT_AND_BACK, GL_LINE );
		}
		else
		{
			glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
		}

		if( gLight )
		{
			glEnable( GL_LIGHTING );
		}
		else
		{
			glDisable( GL_LIGHTING );
		}
	}
	return res;
}



// ***************************************************************************************
void FACE_Cleanup3D()
{
	glfwKillWnd( g_FACE.hwnd );
}


/*
// ***************************************************************************************
bool FACE_SwitchRes( int * x, int * y, UINT * uWidth, UINT * uHeight )
{
	return true;
}
*/

/*
// ***************************************************************************************
void FACE_RestoreRes()
{

}
*/
